import { useEffect, useState } from 'react';
import { Dialog, DialogProps } from '../../components/layout/Dialog';
import React from 'react';
import { useAppDispatch, useSchemaSettings } from '../../data-access';
import { SchemaConnectionTypes, SchemaSettings, schemaConnectionTypeArray, setSchemaSettings } from '../../data-access/store/schemaSlice';
import { FormActions, FormBody, FormCard, FormControl, FormHBar, FormTitle, FormDiv } from '../../components/forms';
import { Layouts } from '../../graph-layout';
import { Input } from '../../components/inputs';

export const SchemaDialog = (props: DialogProps) => {
  const settings = useSchemaSettings();
  const dispatch = useAppDispatch();
  const [state, setState] = React.useState<SchemaSettings>(settings);

  useEffect(() => {
    setState(settings);
  }, [settings, props.open]);

  function submit() {
    dispatch(setSchemaSettings(state));
    props.onClose();
  }

  return (
    <form
      className="w-full"
      onSubmit={(e) => {
        e.preventDefault();
        submit();
      }}
    >
      <FormControl>
        <Input
          type="dropdown"
          label="Type of Connection"
          value={state.connectionType}
          options={schemaConnectionTypeArray}
          onChange={(value: string | number) => {
            setState({ ...state, connectionType: value as SchemaConnectionTypes });
          }}
        />
      </FormControl>
      <FormHBar />
      <FormControl>
        <Input
          type="boolean"
          value={state.animatedEdges}
          label="Animated Edges"
          onChange={(value: boolean) => {
            setState({ ...state, animatedEdges: value as any });
          }}
        />
      </FormControl>
      <FormHBar />
      <FormControl>
        <Input
          type="dropdown"
          label="Layout Type"
          value={state.layoutName}
          options={Object.values(Layouts)}
          onChange={(value: string | number) => {
            setState({ ...state, layoutName: value as any });
          }}
        />
      </FormControl>
      <FormHBar />

      <FormActions onClose={props.onClose} />
    </form>
  );
};
