/**
 * This program has been developed by a student from the bachelor Information Science at
 * Utrecht University within the Research Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

import { CompositeLayer } from 'deck.gl/typed';
import { ScatterplotLayer, LineLayer } from '@deck.gl/layers/typed';
import { LayerProps } from '../types';
import { getColorFromScale } from '../utils';
import { BrushingExtension } from '@deck.gl/extensions/typed';
import GeoNode from '../core/node';

export class GeoNodeLinkLayer extends CompositeLayer<LayerProps> {
  /**
   * Basic node-link layer that just visualizes the data
   */
  static componentName = 'Knowledge Graph Layer';

  shouldUpdateState({ changeFlags }: { changeFlags: any }) {
    return changeFlags.somethingChanged;
  }

  updateState({ props }: { props: LayerProps }) {
    const zoom = Math.floor(this.context.viewport.zoom);
    if (zoom !== this.state.zoom) {
      const node_index: { [key: string]: any } = props.graph.getNodes().reduce((obj: { [key: string]: any }, item: GeoNode) => {
        obj[item.id] = { coordinates: [item.lon, item.lat] };
        return obj;
      }, {});

      this.setState(zoom);
    }
  }

  renderLayers() {
    const { graph, colorScale, brushing } = this.props;

    return [
      new LineLayer({
        id: 'edges',
        data: graph.getEdges(),
        pickable: true,
        //@ts-ignore
        extensions: [new BrushingExtension()],
        brushingEnabled: brushing,
        brushingRadius: 500000,
        brushingTarget: 'source_target',
        getWidth: (d: any) => d.count,
        getSourcePosition: (d: any) => d.source,
        getTargetPosition: (d: any) => d.target,
        getColor: (d: any) => getColorFromScale(d.data.prediction, colorScale),
      }),
      new ScatterplotLayer({
        id: 'nodes',
        data: graph.getNodes(),
        pickable: true,
        opacity: 0.6,
        interactive: true,
        filled: true,
        radiusScale: 3,
        radiusMinPixels: 3,
        radiusMaxPixels: 20,
        getPosition: (d: any) => d.getCoordinates(),
        getFillColor: [250, 0, 0],
      }),
    ];
  }
}
