/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/

import { select } from 'd3';
import React, { useEffect, useRef } from 'react';
import CustomLine from './CustomLine';

type RowLabelColumnProps = {
  onMouseEnter: (row: number) => void;
  onMouseLeave: (row: number) => void;

  titles: string[];
  width: number;
  rowHeight: number;
  yOffset: number;
};
export const RowLabelColumn = (props: RowLabelColumnProps) => {
  const titleRows = props.titles.map((title, i) => (
    <RowLabel
      key={i}
      index={i}
      title={title}
      width={props.width}
      rowHeight={props.rowHeight}
      yOffset={props.yOffset}
      onMouseEnter={props.onMouseEnter}
      onMouseLeave={props.onMouseLeave}
    />
  ));

  return (
    <g>
      {titleRows}
      <g>
        <CustomLine x1={0} x2={0} y1={props.yOffset} y2={props.titles.length * props.rowHeight + props.yOffset} strokeWidth={1} />
        <CustomLine
          x1={props.width}
          x2={props.width}
          y1={props.yOffset}
          y2={props.titles.length * props.rowHeight + props.yOffset}
          strokeWidth={1}
        />
      </g>
    </g>
  );
};

type RowLabelProps = {
  onMouseEnter: (row: number) => void;
  onMouseLeave: (row: number) => void;

  title: string;
  width: number;
  rowHeight: number;
  index: number;
  yOffset: number;
};

const RowLabel = (props: RowLabelProps) => {
  const ref = useRef<SVGGElement>(null);

  useEffect(() => {
    if (ref.current === null) return;
    select(ref.current)
      .on('mouseover', () => props.onMouseEnter(props.index))
      .on('mouseout', () => props.onMouseLeave(props.index));
  }, [ref.current]);

  return (
    <g
      ref={ref}
      className={'rowsLabel row-' + props.index}
      transform={'translate(0,' + (props.yOffset + props.index * props.rowHeight) + ')'}
    >
      <rect
        width={props.width}
        height={props.rowHeight}
        fill={props.index % 2 === 0 ? 'hsl(var(--clr-sec--50))' : 'hsl(var(--clr-sec--0))'}
        strokeWidth={0}
      ></rect>
      {props.index === 0 && <CustomLine x1={0} x2={props.width} y1={0} y2={0} strokeWidth={1} />}
      <CustomLine x1={0} x2={props.width} y1={props.rowHeight} y2={props.rowHeight} strokeWidth={1} />
      <text
        x={props.width * 0.1}
        y={props.rowHeight / 2}
        dy="0"
        fill="hsl(var(--clr-sec--800))"
        dominantBaseline="middle"
        style={{ stroke: 'none' }}
      >
        {props.title}
      </text>
    </g>
  );
};
